@@ -43,6 +43,7 @@ class hotspot_backend:
             'ipt_MASQUERADE', # PONDER: Do we need to insert this module or will iptables take care of it ?
         ]
         self.wlanif = 'wlan0'
+        self.usbif = 'usb0'
         self.gprsif = 'gprs0'
 
         # Modules directory
@@ -86,6 +87,15 @@ class hotspot_backend:
                 # Have atleast somewhat sane value here
                 self.network = '10.10.10.'
 
+        self.interface = self.gc.get_string('/apps/mobilehotspot/interface_name')
+        if self.interface == 'USB':
+            self.interface_name = self.usbif
+        elif self.interface == 'WLAN':
+            self.interface_name = self.wlanif
+        else:
+            self.interface = 'WLAN'
+            self.interface_name = self.wlanif
+
         # Currently only WEP is supported, so we use this as encryption enabled/disabled indicator
         self.encryption_algo = self.gc.get_string('/apps/mobilehotspot/encryption_algo')
         self.encryption_key = self.gc.get_string('/apps/mobilehotspot/encryption_key')
@@ -129,12 +139,35 @@ class hotspot_backend:
         """start the hotspot backend"""
         #self.start_stop_iphbd()
         self.start_insert_modules()
+        if self.interface == 'USB':
+            #return self.check_usb_mode()
+            self.check_usb_mode()
+            self.start_configure_usb()
+        else:
+            self.start_configure_wlan()
         self.start_open_connection()
-        self.start_configure_wlan()
-        self.start_run_dnsmasq()
+        self.start_run_dnsmasq(self.interface_name)
         self.start_setup_NAT()
         self.store_current_state()
 
+    def check_usb_mode(self):
+        usb_mode = helpers.exec_cmd('cat /proc/driver/musb_hdrc | grep "Gadget driver:" | cut -d" " -f 3')
+        if usb_mode != 'g_nokia':
+            note = 'USB must be in PC Suite Mode'
+            helpers.exec_cmd('dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteDialog string:"' + note + '" uint32:0 string:"OK"')
+
+    def start_configure_usb(self):
+        helpers.exec_cmd('ifconfig ' + self.usbif + ' ' + self.network + '1 netmask 255.255.255.0 up')
+        # TODO: Verify that the interface is fully up, now we just sleep a moment
+        time.sleep(1)
+
+    def stop_configure_usb(self):
+        """Takes down interface"""
+        # It seems we need to take the wlan interface back to managed mode or there will be trouble
+        helpers.exec_cmd('ifconfig ' + self.usbif + ' down')
+        # TODO: Verify that device is actually configured and properly down, for now we just sleep a moment
+        time.sleep(1)
+
     def start_insert_modules(self):
         """Calls test_insert_module() for each needed module"""
         for modulename in self.needed_modules:
@@ -246,12 +279,12 @@ class hotspot_backend:
         # TODO: Verify that the interface is fully up, now we just sleep a moment
         time.sleep(3)
 
-    def start_run_dnsmasq(self):
+    def start_run_dnsmasq(self, interface_name):
         """Starts the dnsmasq daemon with dhcp enabled"""
         dnsmasq_command = '/usr/sbin/dnsmasq'
-        dnsmasq_options = ' -i ' + self.wlanif
+        dnsmasq_options = ' -i ' + interface_name
         dnsmasq_options += ' -a ' + self.network + '1'
-        dnsmasq_options += ' -I lo -z -x /var/run/dnsmasq.' + self.wlanif + '.pid'
+        dnsmasq_options += ' -I lo -z -x /var/run/dnsmasq.' + interface_name + '.pid'
         dnsmasq_options += ' --dhcp-range=' + self.network + '10,' + self.network + '100,6h'
         dnsmasq_options += ' --dhcp-option=3,' + self.network + '1'
         dnsmasq_options += ' --dhcp-option=6,' + self.network + '1'
@@ -282,14 +315,17 @@ class hotspot_backend:
         helpers.exec_cmd('iptables -P INPUT ACCEPT')
         helpers.exec_cmd('iptables -P OUTPUT ACCEPT')
 
-        helpers.exec_cmd('iptables --table nat --append POSTROUTING --out-interface ' + self.gprsif + ' -j MASQUERADE')
+        helpers.exec_cmd('iptables -t nat -A POSTROUTING -j MASQUERADE -s ' + self.network + '0/24')
 
     def stop(self):
         """stop the hotspot backend"""
         try:
             self.stop_teardown_NAT()
-            self.stop_stop_dnsmasq()
-            self.stop_configure_wlan()
+            self.stop_stop_dnsmasq(self.interface_name)
+            if self.interface == 'USB':
+                self.stop_configure_usb()
+            else:
+                self.stop_configure_wlan()
             self.stop_close_connection()
             self.stop_remove_modules()
             #self.stop_start_iphbd()
@@ -312,9 +348,9 @@ class hotspot_backend:
         import distutils.file_util
         distutils.file_util.write_file('/proc/sys/net/ipv4/ip_forward', [ '0', ])
 
-    def stop_stop_dnsmasq(self):
+    def stop_stop_dnsmasq(self, interface_name):
         """Stops the dnsmasq daemon previously started"""
-        pidfile = '/var/run/dnsmasq.' + self.wlanif + '.pid'
+        pidfile = '/var/run/dnsmasq.' + interface_name + '.pid'
         import os
         if (os.path.exists(pidfile)):
             import signal
