@@ -30,6 +30,6 @@ def verify_wep_key(key_bytes):
 
 def hex_encode(input_str):
     output_str = ''
-    for char in output_str:
+    for char in input_str:
         output_str = output_str + '%X' % ord(char)
     return output_str
